# Python
__pycache__/
*.py[cod]
*$py.class
*.so

# Virtual Environment
.venv/
venv/
ENV/
env/

# IDEs
.vscode/
.idea/
*.swp
*.swo

# OS
.DS_Store
Thumbs.db

# Git
.git/
.gitignore

# Environment variables
.env
.env.local

# Documentation
*.md
!README.md

# Testing
tests/
.coverage
htmlcov/
.pytest_cache/

# Development files
.editorconfig
